@@ -20,13 +20,9 @@
 #ifndef __HAVE_ACTIONS_H__
 #define __HAVE_ACTIONS_H__
 
-#include "screenshooter-utils.h"
-#include "screenshooter-capture.h"
 #include "screenshooter-global.h"
-#include "screenshooter-dialogs.h"
-#include "screenshooter-imgur.h"
 
-gboolean screenshooter_take_screenshot_idle (ScreenshotData *sd);
-gboolean screenshooter_action_idle          (ScreenshotData *sd);
+void screenshooter_take_screenshot      (ScreenshotData *sd,
+                                         gboolean immediate);
 
 #endif
