@@ -329,7 +329,7 @@ static GdkPixbuf
 
   root = gdk_get_default_root_window ();
 
-  if (has_extents = screenshooter_get_gtk_frame_extents (window, &extents))
+  if ((has_extents = screenshooter_get_gtk_frame_extents (window, &extents)))
     border = FALSE;
 
   if (border)
@@ -670,13 +670,15 @@ static gboolean cb_size_window_draw (GtkWidget *widget,
 
 static void size_window_get_offset (GtkWidget *widget,
                                     gint digits,
+                                    gint digit_width,
+                                    gint line_height,
                                     gint x_event,
                                     gint y_event,
                                     gint *x_offset,
                                     gint *y_offset)
 {
   GdkRectangle geometry;
-  gint relative_x, relative_y;
+  gint relative_x, relative_y, text_width;
 
   GdkDisplay *display = gtk_widget_get_display (widget);
   GdkMonitor *monitor = gdk_display_get_monitor_at_point (display, x_event, y_event);
@@ -688,11 +690,16 @@ static void size_window_get_offset (GtkWidget *widget,
   *x_offset = -2;
   *y_offset = -4;
 
-  if (relative_x > geometry.width - (digits * 9))
-    *x_offset -= digits * 9;
+  /* Add 3/4 of a digit width as right padding */
+  text_width = (digits + 0.75) * digit_width;
+  /* Add 10% of line height as bottom padding */
+  line_height = line_height * 1.1;
 
-  if (relative_y > geometry.height - 20)
-    *y_offset -= 20;
+  if (relative_x > geometry.width - text_width)
+    *x_offset -= text_width;
+
+  if (relative_y > geometry.height - line_height)
+    *y_offset -= line_height;
 }
 
 
@@ -746,6 +753,7 @@ static gboolean cb_motion_notify (GtkWidget *widget,
 {
   gchar *coords;
   gint rect_width, rect_height, x_offset, y_offset;
+  static gint digit_width = -1, line_height = -1;
 
   if (rbdata->left_pressed)
     {
@@ -813,7 +821,26 @@ static gboolean cb_motion_notify (GtkWidget *widget,
 
       coords = g_strdup_printf ("%d x %d", rect_width, rect_height);
 
+      if (digit_width == -1)
+        {
+          PangoLayout *layout;
+          PangoContext *context;
+          PangoFontMetrics *metrics;
+
+          layout = gtk_label_get_layout (GTK_LABEL (rbdata->size_label));
+          context = pango_layout_get_context (layout);
+          metrics = pango_context_get_metrics (context,
+                                               pango_context_get_font_description (context),
+                                               pango_context_get_language (context));
+
+          digit_width = PANGO_PIXELS_CEIL (pango_font_metrics_get_approximate_digit_width (metrics));
+          line_height = PANGO_PIXELS_CEIL (pango_font_metrics_get_height (metrics));
+
+          pango_font_metrics_unref (metrics);
+        }
+
       size_window_get_offset (rbdata->size_window, strlen (coords),
+                              digit_width, line_height,
                               event->x, event->y,
                               &x_offset, &y_offset);
 
@@ -1274,6 +1301,7 @@ static GdkPixbuf
 
   Display *display;
   gint screen;
+  gint scale;
   RbData rbdata;
   GdkCursor *xhair_cursor;
   GdkGrabStatus res;
@@ -1285,6 +1313,7 @@ static GdkPixbuf
   root_window = gdk_get_default_root_window ();
   display = gdk_x11_get_default_xdisplay ();
   screen = gdk_x11_get_default_screen ();
+  scale = gdk_window_get_scale_factor (root_window);
 
   /* Change cursor to cross-hair */
   TRACE ("Set the cursor");
@@ -1353,11 +1382,10 @@ static GdkPixbuf
   if (G_LIKELY (!rbdata.cancelled))
     {
       TRACE ("Get the pixbuf for the screenshot");
-
-      screenshot = capture_rectangle_screenshot (rbdata.rectangle.x,
-                                                 rbdata.rectangle.y,
-                                                 rbdata.rectangle.width,
-                                                 rbdata.rectangle.height,
+      screenshot = capture_rectangle_screenshot (rbdata.rectangle.x / scale,
+                                                 rbdata.rectangle.y / scale,
+                                                 rbdata.rectangle.width / scale,
+                                                 rbdata.rectangle.height / scale,
                                                  delay, show_mouse);
     }
 
@@ -1377,10 +1405,13 @@ static GdkPixbuf
 
 /**
  * screenshooter_capture_screenshot:
- * @region: the region to be screenshoted. It can be FULLSCREEN,
+ * @region: the region to be captured. It can be FULLSCREEN,
  *          ACTIVE_WINDOW or SELECT.
  * @delay: the delay before the screenshot is captured, in seconds.
- * @mouse: whether the mouse pointer should be displayed on the screenshot.
+ * @show_mouse: whether the mouse pointer should be displayed on the
+ *              screenshot.
+ * @show_border: whether the window border should be displayed on the
+ *               screenshot.
  *
  * Captures a screenshot with the given options. If @region is FULLSCREEN,
  * the screenshot is captured after @delay seconds. If @region is
@@ -1391,6 +1422,9 @@ static GdkPixbuf
  *
  * @show_mouse is only taken into account when @region is FULLSCREEN
  * or ACTIVE_WINDOW.
+ * 
+ * @show_border is only taken into account when @region is ACTIVE_WINDOW
+ * and if the window uses server side decorations.
  *
  * Return value: a #GdkPixbuf containing the screenshot or %NULL
  * (if @region is SELECT, the user can cancel the operation).
@@ -1398,6 +1432,7 @@ static GdkPixbuf
 GdkPixbuf *screenshooter_capture_screenshot (gint     region,
                                              gint     delay,
                                              gboolean show_mouse,
+                                             gboolean show_border,
                                              gboolean plugin)
 {
   GdkPixbuf *screenshot = NULL;
@@ -1428,7 +1463,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
       TRACE ("Get the screenshot of the active window");
       window = screenshooter_get_active_window (screen, &needs_unref, &border);
-      screenshot = get_window_screenshot (window, show_mouse, border);
+      screenshot = get_window_screenshot (window, show_mouse, show_border && border);
       if (needs_unref)
         g_object_unref (window);
     }
