@@ -44,18 +44,21 @@ typedef struct
   gint region;
   gint show_save_dialog;
   gint show_mouse;
+  gint show_border;
   gint delay;
   gint action;
   gboolean plugin;
   gboolean action_specified;
   gboolean region_specified;
   gboolean timestamp;
+  gboolean enable_imgur_upload;
   gboolean path_is_dir;
   gchar *screenshot_dir;
   gchar *title;
   gchar *app;
   GAppInfo *app_info;
   gchar *last_user;
+  gchar *last_extension;
   GdkPixbuf *screenshot;
 }
 ScreenshotData;
