@@ -81,6 +81,27 @@ get_active_window_from_xlib (void)
 
 
 
+/**
+ * @format - a date format string
+ *
+ * Builds a timestamp using local time.
+ * Returned string should be released with g_free()
+ **/
+static gchar *
+get_datetime (const gchar *format)
+{
+  gchar *timestamp;
+  GDateTime *now = g_date_time_new_now_local ();
+  timestamp = g_date_time_format (now, format);
+
+  g_date_time_unref (now);
+  /* TODO: strip potential : and / if the format is configurable */
+  DBG ("datetime is %s", timestamp);
+  return timestamp;
+}
+
+
+
 /* Public */
 
 
@@ -119,11 +140,14 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
   gint region = FULLSCREEN;
   gint action = SAVE;
   gint show_mouse = 1;
+  gint show_border = 1;
   gboolean timestamp = TRUE;
   gchar *screenshot_dir = g_strdup (default_uri);
   gchar *title = g_strdup (_("Screenshot"));
   gchar *app = g_strdup ("none");
   gchar *last_user = g_strdup ("");
+  gchar *last_extension = g_strdup ("png");
+  gboolean enable_imgur_upload = TRUE;
 
   if (G_LIKELY (file != NULL))
     {
@@ -139,7 +163,9 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
           region = xfce_rc_read_int_entry (rc, "region", FULLSCREEN);
           action = xfce_rc_read_int_entry (rc, "action", SAVE);
           show_mouse = xfce_rc_read_int_entry (rc, "show_mouse", 1);
+          show_border = xfce_rc_read_int_entry (rc, "show_border", 1);
           timestamp = xfce_rc_read_bool_entry (rc, "timestamp", TRUE);
+          enable_imgur_upload = xfce_rc_read_bool_entry (rc, "enable_imgur_upload", TRUE);
 
           g_free (app);
           app = g_strdup (xfce_rc_read_entry (rc, "app", "none"));
@@ -147,6 +173,9 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
           g_free (last_user);
           last_user = g_strdup (xfce_rc_read_entry (rc, "last_user", ""));
 
+          g_free (last_extension);
+          last_extension = g_strdup (xfce_rc_read_entry (rc, "last_extension", "png"));
+
           g_free (screenshot_dir);
           screenshot_dir =
             g_strdup (xfce_rc_read_entry (rc, "screenshot_dir", default_uri));
@@ -168,12 +197,15 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
   sd->region = region;
   sd->action = action;
   sd->show_mouse = show_mouse;
+  sd->show_border = show_border;
   sd->timestamp = timestamp;
   sd->screenshot_dir = screenshot_dir;
   sd->title = title;
   sd->app = app;
   sd->app_info = NULL;
   sd->last_user = last_user;
+  sd->last_extension = last_extension;
+  sd->enable_imgur_upload = enable_imgur_upload;
 }
 
 
@@ -199,7 +231,9 @@ screenshooter_write_rc_file (const gchar *file, ScreenshotData *sd)
 
   xfce_rc_write_entry (rc, "app", sd->app);
   xfce_rc_write_entry (rc, "last_user", sd->last_user);
+  xfce_rc_write_entry (rc, "last_extension", sd->last_extension);
   xfce_rc_write_entry (rc, "screenshot_dir", sd->screenshot_dir);
+  xfce_rc_write_bool_entry (rc, "enable_imgur_upload", sd->enable_imgur_upload);
 
   /* do not save if action was specified from cli */
   if (!sd->action_specified)
@@ -213,6 +247,7 @@ screenshooter_write_rc_file (const gchar *file, ScreenshotData *sd)
     xfce_rc_write_int_entry (rc, "delay", sd->delay);
     xfce_rc_write_int_entry (rc, "region", sd->region);
     xfce_rc_write_int_entry (rc, "show_mouse", sd->show_mouse);
+    xfce_rc_write_int_entry (rc, "show_border", sd->show_border);
   }
 
   TRACE ("Flush and close the rc file");
@@ -351,23 +386,84 @@ void screenshooter_error (const gchar *format, ...)
   g_free (message);
 }
 
-/**
- * screenshooter_get_datetime
- * @format - a date format string
- *
- * Builds a timestamp using local time.
- * Returned string should be released with g_free()
- **/
-gchar *screenshooter_get_datetime (const gchar *format)
-{
-  gchar *timestamp;
-  GDateTime *now = g_date_time_new_now_local();
-  timestamp = g_date_time_format (now, format);
 
-  g_date_time_unref (now);
-  /* TODO: strip potential : and / if the format is configurable */
-  DBG("datetime is %s", timestamp);
-  return timestamp;
+
+/* If @timestamp is true, generates a file name @title - date - hour - n.png,
+ * where n is the lowest integer such as this file does not exist in the @uri
+ * folder.
+ * Else, generates a file name @title-n.png, where n is the lowest integer
+ * such as this file does not exist in the @uri folder.
+ *
+ * @uri: uri of the folder for which the filename should be generated.
+ * @title: the main title of the file name.
+ * @timestamp: whether the date and the hour should be appended to the file name.
+ *
+ * returns: the filename or NULL if *uri == NULL.
+*/
+gchar *
+screenshooter_get_filename_for_uri (const gchar *uri,
+                                    const gchar *title,
+                                    const gchar *extension,
+                                    gboolean     timestamp)
+{
+  gboolean exists = TRUE;
+  GFile *directory;
+  GFile *file;
+  gchar *base_name;
+  gchar *datetime;
+  const gchar *strftime_format = "%Y-%m-%d_%H-%M-%S";
+
+  gint i;
+
+  if (G_UNLIKELY (uri == NULL))
+    {
+      TRACE ("URI was NULL");
+      return NULL;
+    }
+
+  TRACE ("Get the folder corresponding to the URI");
+  datetime = get_datetime (strftime_format);
+  directory = g_file_new_for_uri (uri);
+  if (!timestamp)
+    base_name = g_strconcat (title, ".", extension, NULL);
+  else
+    base_name = g_strconcat (title, "_", datetime, ".", extension, NULL);
+
+  file = g_file_get_child (directory, base_name);
+
+  if (!g_file_query_exists (file, NULL))
+    {
+      g_object_unref (file);
+      g_object_unref (directory);
+
+      return base_name;
+    }
+
+  g_object_unref (file);
+  g_free (base_name);
+
+  for (i = 1; exists; ++i)
+    {
+      const gchar *suffix = g_strdup_printf ("-%d.%s", i, extension);
+
+      if (!timestamp)
+         base_name = g_strconcat (title, suffix, NULL);
+       else
+         base_name = g_strconcat (title, "_", datetime, suffix, NULL);
+
+      file = g_file_get_child (directory, base_name);
+      exists = g_file_query_exists (file, NULL);
+
+      if (exists)
+        g_free (base_name);
+
+      g_object_unref (file);
+    }
+
+  g_free (datetime);
+  g_object_unref (directory);
+
+  return base_name;
 }
 
 
