@@ -33,6 +33,7 @@ gboolean window = FALSE;
 gboolean region = FALSE;
 gboolean fullscreen = FALSE;
 gboolean mouse = FALSE;
+gboolean no_border = FALSE;
 gboolean clipboard = FALSE;
 gboolean upload_imgur = FALSE;
 gchar *screenshot_dir = NULL;
@@ -64,6 +65,11 @@ static GOptionEntry entries[] =
     N_("Display the mouse on the screenshot"),
     NULL
   },
+  {
+    "no-border", 0, G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &no_border,
+    N_("Removes the window border from the screenshot"),
+    NULL
+  },
   {
     "open", 'o', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &application,
     N_("Application to open the screenshot"),
@@ -78,7 +84,7 @@ static GOptionEntry entries[] =
   },
   {
     "save", 's', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_FILENAME, &screenshot_dir,
-    N_("File path or directory where the screenshot will be saved"),
+    N_("File path or directory where the screenshot will be saved, accepts png, jpg and bmp extensions"),
     NULL
   },
   {
@@ -270,6 +276,9 @@ int main (int argc, char **argv)
       /* Whether to display the mouse pointer on the screenshot */
       mouse ? (sd->show_mouse = 1) : (sd->show_mouse = 0);
 
+      /* Whether to display the window border on the screenshot */
+      no_border ? (sd->show_border = 0) : (sd->show_border = 1);
+
       sd->delay = delay;
 
       if (application != NULL)
@@ -343,6 +352,7 @@ int main (int argc, char **argv)
   g_free (sd->title);
   g_free (sd->app);
   g_free (sd->last_user);
+  g_free (sd->last_extension);
   g_free (sd);
 
   TRACE ("Ciao");
